@@ -1,11 +1,13 @@
 """Tests for the redis orm"""
-
-from typing import Dict, Any, List
+from collections import namedtuple
+from typing import Dict, Any
 
 import pytest
 
-from pydantic_redis.config import RedisConfig
-from pydantic_redis.model import Model, strip_leading, NESTED_MODEL_PREFIX
+from pydantic_redis.shared.config import RedisConfig
+from pydantic_redis.shared.model.prop_utils import NESTED_MODEL_PREFIX
+from pydantic_redis.shared.utils import strip_leading
+from pydantic_redis.syncio.model import Model
 from test.conftest import (
     redis_store_fixture,
     Book,
@@ -13,7 +15,6 @@ from test.conftest import (
     Author,
     authors,
     Library,
-    libraries,
 )
 
 
@@ -206,6 +207,27 @@ def test_select_default(store):
     assert sorted_books == sorted_response
 
 
+@pytest.mark.parametrize("store", redis_store_fixture)
+def test_select_default_paginated(store):
+    """
+    Selecting without arguments returns the book models after
+    skipping `skip` number of models and returning upto `limit` number of items
+    """
+    Book.insert(books)
+    Record = namedtuple("Record", ["skip", "limit", "expected"])
+    test_data = [
+        Record(0, 2, sorted(books[:2], key=lambda x: x.title)),
+        Record(None, 2, sorted(books[:2], key=lambda x: x.title)),
+        Record(2, 2, sorted(books[2:4], key=lambda x: x.title)),
+        Record(3, 2, sorted(books[3:5], key=lambda x: x.title)),
+        Record(0, 3, sorted(books[:3], key=lambda x: x.title)),
+    ]
+    for record in test_data:
+        response = Book.select(skip=record.skip, limit=record.limit)
+        sorted_response = sorted(response, key=lambda x: x.title)
+        assert record.expected == sorted_response
+
+
 @pytest.mark.parametrize("store", redis_store_fixture)
 def test_select_some_columns(store):
     """
@@ -214,7 +236,7 @@ def test_select_some_columns(store):
     Book.insert(books)
     books_dict = {book.title: book for book in books}
     columns = ["title", "author", "in_stock"]
-    response = Book.select(columns=["title", "author", "in_stock"])
+    response = Book.select(columns=columns)
     response_dict = {book["title"]: book for book in response}
 
     for title, book in books_dict.items():
@@ -229,6 +251,41 @@ def test_select_some_columns(store):
                 assert f"{book_in_response[column]}" == f"{getattr(book, column)}"
 
 
+@pytest.mark.parametrize("store", redis_store_fixture)
+def test_select_some_columns_paginated(store):
+    """
+    Selecting some columns returns a list of dictionaries of all books models with only those columns
+    skipping `skip` number of models and returning upto `limit` number of items
+    """
+    Book.insert(books)
+    columns = ["title", "author", "in_stock"]
+
+    Record = namedtuple("Record", ["skip", "limit", "expected"])
+    test_data = [
+        Record(0, 2, sorted(books[:2], key=lambda x: x.title)),
+        Record(None, 2, sorted(books[:2], key=lambda x: x.title)),
+        Record(2, 2, sorted(books[2:4], key=lambda x: x.title)),
+        Record(3, 2, sorted(books[3:5], key=lambda x: x.title)),
+        Record(0, 3, sorted(books[:3], key=lambda x: x.title)),
+    ]
+    for record in test_data:
+        response = Book.select(columns=columns, skip=record.skip, limit=record.limit)
+        response_dict = {book["title"]: book for book in response}
+        books_dict = {book.title: book for book in record.expected}
+        assert len(record.expected) == len(response_dict)
+
+        for title, book in books_dict.items():
+            book_in_response = response_dict[title]
+            assert isinstance(book_in_response, dict)
+            assert sorted(book_in_response.keys()) == sorted(columns)
+
+            for column in columns:
+                if column == "author":
+                    assert book_in_response[column] == getattr(book, column)
+                else:
+                    assert f"{book_in_response[column]}" == f"{getattr(book, column)}"
+
+
 @pytest.mark.parametrize("store", redis_store_fixture)
 def test_select_some_ids(store):
     """
